@@ -48,6 +48,14 @@ mlir::Value fir::runtime::genGetCommand(fir::FirOpBuilder &builder,
   return builder.create<fir::CallOp>(loc, runtimeFunc, args).getResult(0);
 }
 
+mlir::Value fir::runtime::genGetPID(fir::FirOpBuilder &builder,
+                                    mlir::Location loc) {
+  auto runtimeFunc =
+      fir::runtime::getRuntimeFunc<mkRTKey(GetPID)>(loc, builder);
+
+  return builder.create<fir::CallOp>(loc, runtimeFunc).getResult(0);
+}
+
 mlir::Value fir::runtime::genGetCommandArgument(
     fir::FirOpBuilder &builder, mlir::Location loc, mlir::Value number,
     mlir::Value value, mlir::Value length, mlir::Value errmsg) {
@@ -63,32 +71,20 @@ mlir::Value fir::runtime::genGetCommandArgument(
   return builder.create<fir::CallOp>(loc, runtimeFunc, args).getResult(0);
 }
 
-mlir::Value fir::runtime::genEnvVariableValue(
-    fir::FirOpBuilder &builder, mlir::Location loc, mlir::Value name,
-    mlir::Value value, mlir::Value trimName, mlir::Value errmsg) {
-  auto valueFunc =
-      fir::runtime::getRuntimeFunc<mkRTKey(EnvVariableValue)>(loc, builder);
-  mlir::FunctionType valueFuncTy = valueFunc.getFunctionType();
+mlir::Value fir::runtime::genGetEnvVariable(fir::FirOpBuilder &builder,
+                                            mlir::Location loc,
+                                            mlir::Value name, mlir::Value value,
+                                            mlir::Value length,
+                                            mlir::Value trimName,
+                                            mlir::Value errmsg) {
+  auto runtimeFunc =
+      fir::runtime::getRuntimeFunc<mkRTKey(GetEnvVariable)>(loc, builder);
+  mlir::FunctionType runtimeFuncTy = runtimeFunc.getFunctionType();
   mlir::Value sourceFile = fir::factory::locationToFilename(builder, loc);
   mlir::Value sourceLine =
-      fir::factory::locationToLineNo(builder, loc, valueFuncTy.getInput(5));
-  llvm::SmallVector<mlir::Value> args =
-      fir::runtime::createArguments(builder, loc, valueFuncTy, name, value,
-                                    trimName, errmsg, sourceFile, sourceLine);
-  return builder.create<fir::CallOp>(loc, valueFunc, args).getResult(0);
-}
-
-mlir::Value fir::runtime::genEnvVariableLength(fir::FirOpBuilder &builder,
-                                               mlir::Location loc,
-                                               mlir::Value name,
-                                               mlir::Value trimName) {
-  auto lengthFunc =
-      fir::runtime::getRuntimeFunc<mkRTKey(EnvVariableLength)>(loc, builder);
-  mlir::FunctionType lengthFuncTy = lengthFunc.getFunctionType();
-  mlir::Value sourceFile = fir::factory::locationToFilename(builder, loc);
-  mlir::Value sourceLine =
-      fir::factory::locationToLineNo(builder, loc, lengthFuncTy.getInput(3));
+      fir::factory::locationToLineNo(builder, loc, runtimeFuncTy.getInput(6));
   llvm::SmallVector<mlir::Value> args = fir::runtime::createArguments(
-      builder, loc, lengthFuncTy, name, trimName, sourceFile, sourceLine);
-  return builder.create<fir::CallOp>(loc, lengthFunc, args).getResult(0);
+      builder, loc, runtimeFuncTy, name, value, length, trimName, errmsg,
+      sourceFile, sourceLine);
+  return builder.create<fir::CallOp>(loc, runtimeFunc, args).getResult(0);
 }
