@@ -8,13 +8,17 @@
 
 ! Fortran 2018 Clause 17
 
-module ieee_arithmetic
-  ! 17.1: "The module IEEE_ARITHMETIC behaves as if it contained a
-  ! USE statement for IEEE_EXCEPTIONS; everything that is public in
-  ! IEEE_EXCEPTIONS is public in IEEE_ARITHMETIC."
-  use __Fortran_ieee_exceptions
+include '../include/flang/Runtime/magic-numbers.h'
 
-  use __Fortran_builtins, only: &
+module ieee_arithmetic
+  ! F18 Clause 17.1p1:
+  ! The module IEEE_ARITHMETIC behaves as if it contained a USE statement for
+  ! IEEE_EXCEPTIONS; everything that is public in IEEE_EXCEPTIONS is public in
+  ! IEEE_ARITHMETIC.
+  use __fortran_ieee_exceptions
+
+  use __fortran_builtins, only: &
+    ieee_fma => __builtin_fma, &
     ieee_is_nan => __builtin_ieee_is_nan, &
     ieee_is_negative => __builtin_ieee_is_negative, &
     ieee_is_normal => __builtin_ieee_is_normal, &
@@ -42,21 +46,25 @@ module ieee_arithmetic
   end type ieee_class_type
 
   type(ieee_class_type), parameter :: &
-    ieee_signaling_nan = ieee_class_type(1), &
-    ieee_quiet_nan = ieee_class_type(2), &
-    ieee_negative_inf = ieee_class_type(3), &
-    ieee_negative_normal = ieee_class_type(4), &
-    ieee_negative_denormal = ieee_class_type(5), &
-    ieee_negative_zero = ieee_class_type(6), &
-    ieee_positive_zero = ieee_class_type(7), &
-    ieee_positive_subnormal = ieee_class_type(8), &
-    ieee_positive_normal = ieee_class_type(9), &
-    ieee_positive_inf = ieee_class_type(10), &
-    ieee_other_value = ieee_class_type(11)
+    ieee_signaling_nan = ieee_class_type(_FORTRAN_RUNTIME_IEEE_SIGNALING_NAN), &
+    ieee_quiet_nan = ieee_class_type(_FORTRAN_RUNTIME_IEEE_QUIET_NAN), &
+    ieee_negative_inf = ieee_class_type(_FORTRAN_RUNTIME_IEEE_NEGATIVE_INF), &
+    ieee_negative_normal = &
+        ieee_class_type(_FORTRAN_RUNTIME_IEEE_NEGATIVE_NORMAL), &
+    ieee_negative_subnormal = &
+        ieee_class_type(_FORTRAN_RUNTIME_IEEE_NEGATIVE_SUBNORMAL), &
+    ieee_negative_zero = ieee_class_type(_FORTRAN_RUNTIME_IEEE_NEGATIVE_ZERO), &
+    ieee_positive_zero = ieee_class_type(_FORTRAN_RUNTIME_IEEE_POSITIVE_ZERO), &
+    ieee_positive_subnormal = &
+         ieee_class_type(_FORTRAN_RUNTIME_IEEE_POSITIVE_SUBNORMAL), &
+    ieee_positive_normal = &
+        ieee_class_type(_FORTRAN_RUNTIME_IEEE_POSITIVE_NORMAL), &
+    ieee_positive_inf = ieee_class_type(_FORTRAN_RUNTIME_IEEE_POSITIVE_INF), &
+    ieee_other_value = ieee_class_type(_FORTRAN_RUNTIME_IEEE_OTHER_VALUE)
 
   type(ieee_class_type), parameter :: &
-    ieee_negative_subnormal = ieee_negative_denormal, &
-    ieee_positive_denormal = ieee_negative_subnormal
+    ieee_negative_denormal = ieee_negative_subnormal, &
+    ieee_positive_denormal = ieee_positive_subnormal
 
   type :: ieee_round_type
     private
@@ -64,12 +72,12 @@ module ieee_arithmetic
   end type ieee_round_type
 
   type(ieee_round_type), parameter :: &
-    ieee_nearest = ieee_round_type(1), &
-    ieee_to_zero = ieee_round_type(2), &
-    ieee_up = ieee_round_type(3), &
-    ieee_down = ieee_round_type(4), &
-    ieee_away = ieee_round_type(5), &
-    ieee_other = ieee_round_type(6)
+    ieee_to_zero = ieee_round_type(_FORTRAN_RUNTIME_IEEE_TO_ZERO), &
+    ieee_nearest = ieee_round_type(_FORTRAN_RUNTIME_IEEE_NEAREST), &
+    ieee_up = ieee_round_type(_FORTRAN_RUNTIME_IEEE_UP), &
+    ieee_down = ieee_round_type(_FORTRAN_RUNTIME_IEEE_DOWN), &
+    ieee_away = ieee_round_type(_FORTRAN_RUNTIME_IEEE_AWAY), &
+    ieee_other = ieee_round_type(_FORTRAN_RUNTIME_IEEE_OTHER)
 
   interface operator(==)
     elemental logical function ieee_class_eq(x, y)
@@ -220,16 +228,6 @@ module ieee_arithmetic
   PRIVATE_RR(IEEE_COPY_SIGN)
 #undef IEEE_COPY_SIGN_RR
 
-#define IEEE_FMA_R(AKIND) \
-  elemental real(AKIND) function ieee_fma_a##AKIND(a, b, c); \
-    real(AKIND), intent(in) :: a, b, c; \
-  end function ieee_fma_a##AKIND;
-  interface ieee_fma
-    SPECIFICS_R(IEEE_FMA_R)
-  end interface ieee_fma
-  PRIVATE_R(IEEE_FMA)
-#undef IEEE_FMA_R
-
 #define IEEE_GET_ROUNDING_MODE_I(RKIND) \
   subroutine ieee_get_rounding_mode_i##RKIND(round_value, radix); \
     import ieee_round_type; \
@@ -300,6 +298,26 @@ module ieee_arithmetic
   PRIVATE_R(IEEE_LOGB)
 #undef IEEE_LOGB_R
 
+#define IEEE_MAX_R(XKIND) \
+  elemental real(XKIND) function ieee_max_a##XKIND(x, y); \
+    real(XKIND), intent(in) :: x, y; \
+  end function ieee_max_a##XKIND;
+  interface ieee_max
+    SPECIFICS_R(IEEE_MAX_R)
+  end interface ieee_max
+  PRIVATE_R(IEEE_MAX)
+#undef IEEE_MAX_R
+
+#define IEEE_MAX_MAG_R(XKIND) \
+  elemental real(XKIND) function ieee_max_mag_a##XKIND(x, y); \
+    real(XKIND), intent(in) :: x, y; \
+  end function ieee_max_mag_a##XKIND;
+  interface ieee_max_mag
+    SPECIFICS_R(IEEE_MAX_MAG_R)
+  end interface ieee_max_mag
+  PRIVATE_R(IEEE_MAX_MAG)
+#undef IEEE_MAX_MAG_R
+
 #define IEEE_MAX_NUM_R(XKIND) \
   elemental real(XKIND) function ieee_max_num_a##XKIND(x, y); \
     real(XKIND), intent(in) :: x, y; \
@@ -320,6 +338,26 @@ module ieee_arithmetic
   PRIVATE_R(IEEE_MAX_NUM_MAG)
 #undef IEEE_MAX_NUM_MAG_R
 
+#define IEEE_MIN_R(XKIND) \
+  elemental real(XKIND) function ieee_min_a##XKIND(x, y); \
+    real(XKIND), intent(in) :: x, y; \
+  end function ieee_min_a##XKIND;
+  interface ieee_min
+    SPECIFICS_R(IEEE_MIN_R)
+  end interface ieee_min
+  PRIVATE_R(IEEE_MIN)
+#undef IEEE_MIN_R
+
+#define IEEE_MIN_MAG_R(XKIND) \
+  elemental real(XKIND) function ieee_min_mag_a##XKIND(x, y); \
+    real(XKIND), intent(in) :: x, y; \
+  end function ieee_min_mag_a##XKIND;
+  interface ieee_min_mag
+    SPECIFICS_R(IEEE_MIN_MAG_R)
+  end interface ieee_min_mag
+  PRIVATE_R(IEEE_MIN_MAG)
+#undef IEEE_MIN_MAG_R
+
 #define IEEE_MIN_NUM_R(XKIND) \
   elemental real(XKIND) function ieee_min_num_a##XKIND(x, y); \
     real(XKIND), intent(in) :: x, y; \
