@@ -24,6 +24,11 @@ void registerTestFIRAliasAnalysisPass();
 } // namespace test
 } // namespace fir
 
+// Defined in mlir/test, no pulic header.
+namespace mlir {
+void registerSideEffectTestPasses();
+}
+
 int main(int argc, char **argv) {
   fir::support::registerMLIRPassesForFortranTools();
   fir::registerOptCodeGenPasses();
@@ -31,9 +36,10 @@ int main(int argc, char **argv) {
   hlfir::registerHLFIRPasses();
 #ifdef FLANG_INCLUDE_TESTS
   fir::test::registerTestFIRAliasAnalysisPass();
+  mlir::registerSideEffectTestPasses();
 #endif
   DialectRegistry registry;
   fir::support::registerDialects(registry);
   return failed(MlirOptMain(argc, argv, "FIR modular optimizer driver\n",
-      registry, /*preloadDialectsInContext=*/false));
+      registry));
 }
