@@ -6,20 +6,21 @@
 //
 //===----------------------------------------------------------------------===//
 
-#ifndef LIBC_SRC_STRING_ALLOCATING_STRING_UTILS_H
-#define LIBC_SRC_STRING_ALLOCATING_STRING_UTILS_H
+#ifndef LLVM_LIBC_SRC_STRING_ALLOCATING_STRING_UTILS_H
+#define LLVM_LIBC_SRC_STRING_ALLOCATING_STRING_UTILS_H
 
 #include "src/__support/CPP/new.h"
 #include "src/__support/CPP/optional.h"
-#include "src/string/memory_utils/memcpy_implementations.h" // For string_length
+#include "src/__support/macros/config.h"
+#include "src/string/memory_utils/inline_memcpy.h"
 #include "src/string/string_utils.h"
 
 #include <stddef.h> // For size_t
 
-namespace __llvm_libc {
+namespace LIBC_NAMESPACE {
 namespace internal {
 
-cpp::optional<char *> strdup(const char *src) {
+LIBC_INLINE cpp::optional<char *> strdup(const char *src) {
   if (src == nullptr)
     return cpp::nullopt;
   size_t len = string_length(src) + 1;
@@ -32,6 +33,6 @@ cpp::optional<char *> strdup(const char *src) {
 }
 
 } // namespace internal
-} // namespace __llvm_libc
+} // namespace LIBC_NAMESPACE
 
-#endif
+#endif // LLVM_LIBC_SRC_STRING_ALLOCATING_STRING_UTILS_H
