@@ -14,6 +14,7 @@
 
 #include "test_macros.h"
 #include "test_iterators.h"
+#include "test_range.h"
 
 struct SimpleView : std::ranges::view_base {
   int *begin() const;
@@ -48,3 +49,10 @@ static_assert(!std::ranges::__simple_view<WrongConstView>);
 static_assert(!std::ranges::__simple_view<NoConstView>);
 static_assert( std::ranges::__simple_view<DifferentSentinel>);
 static_assert(!std::ranges::__simple_view<WrongConstSentinel>);
+
+// To make sure __simple_view and the test version of the concept stay in sync.
+static_assert(simple_view<SimpleView>);
+static_assert(!simple_view<WrongConstView>);
+static_assert(!simple_view<NoConstView>);
+static_assert(simple_view<DifferentSentinel>);
+static_assert(!simple_view<WrongConstSentinel>);
