@@ -6,7 +6,7 @@
 //
 //===----------------------------------------------------------------------===//
 
-// UNSUPPORTED: c++03
+// UNSUPPORTED: c++03, c++11, c++14
 // UNSUPPORTED: no-localization
 // UNSUPPORTED: no-threads
 // UNSUPPORTED: no-filesystem
@@ -43,8 +43,9 @@
 #include <system_error>
 #include <thread>
 
-#include "filesystem_include.h"
+#include <filesystem>
 #include "filesystem_test_helper.h"
+namespace fs = std::filesystem;
 
 int main(int, char**) {
   scoped_test_env env;
