@@ -20,6 +20,7 @@
 #include <ranges>
 #include <type_traits>
 #include <utility>
+#include "test_macros.h"
 
 // Niebloids, unlike CPOs, are *not* required to be semiregular or even to have
 // a declared type at all; they are specified as "magic" overload sets whose
@@ -64,13 +65,18 @@ static_assert(test(std::ranges::all_of, a, odd));
 static_assert(test(std::ranges::any_of, a, odd));
 static_assert(test(std::ranges::binary_search, a, 42));
 static_assert(test(std::ranges::clamp, 42, 42, 42));
+#if TEST_STD_VER >= 23
+static_assert(test(std::ranges::contains, a, 42));
+#endif
 static_assert(test(std::ranges::copy, a, a));
 static_assert(test(std::ranges::copy_backward, a, a));
 static_assert(test(std::ranges::copy_if, a, a, odd));
 static_assert(test(std::ranges::copy_n, a, 10, a));
 static_assert(test(std::ranges::count, a, 42));
 static_assert(test(std::ranges::count_if, a, odd));
-//static_assert(test(std::ranges::ends_with, a, a));
+#if TEST_STD_VER >= 23
+static_assert(test(std::ranges::ends_with, a, a));
+#endif
 static_assert(test(std::ranges::equal, a, a));
 static_assert(test(std::ranges::equal_range, a, 42));
 static_assert(test(std::ranges::fill, a, 42));
@@ -80,6 +86,10 @@ static_assert(test(std::ranges::find_end, a, a));
 static_assert(test(std::ranges::find_first_of, a, a));
 static_assert(test(std::ranges::find_if, a, odd));
 static_assert(test(std::ranges::find_if_not, a, odd));
+#if TEST_STD_VER >= 23
+static_assert(test(std::ranges::fold_left, a, 0, std::plus()));
+static_assert(test(std::ranges::fold_left_with_iter, a, 0, std::plus()));
+#endif
 static_assert(test(std::ranges::for_each, a, odd));
 static_assert(test(std::ranges::for_each_n, a, 10, odd));
 static_assert(test(std::ranges::generate, a, gen));
@@ -140,7 +150,9 @@ static_assert(test(std::ranges::sort, a));
 static_assert(test(std::ranges::sort_heap, a));
 static_assert(test(std::ranges::stable_partition, a, odd));
 static_assert(test(std::ranges::stable_sort, a));
-//static_assert(test(std::ranges::starts_with, a, a));
+#if TEST_STD_VER > 20
+static_assert(test(std::ranges::starts_with, a, a));
+#endif
 static_assert(test(std::ranges::swap_ranges, a, a));
 static_assert(test(std::ranges::transform, a, a, triple));
 static_assert(test(std::ranges::unique, a));
