@@ -9,16 +9,9 @@
 // NetBSD does not support LC_MONETARY at the moment
 // XFAIL: netbsd
 
-// Failure related to GLIBC's use of U00A0 as mon_thousands_sep
-// and U002E as mon_decimal_point.
-// TODO: U00A0 should be investigated.
-// Possibly related to https://gcc.gnu.org/bugzilla/show_bug.cgi?id=16006
-// XFAIL: linux
-
-// XFAIL: LIBCXX-WINDOWS-FIXME
-
 // REQUIRES: locale.ru_RU.UTF-8
-// XFAIL: LIBCXX-AIX-FIXME
+
+// XFAIL: glibc-old-ru_RU-decimal-point
 
 // <locale>
 
@@ -34,21 +27,9 @@
 #include "test_macros.h"
 #include "test_iterators.h"
 
+#include "locale_helpers.h"
 #include "platform_support.h" // locale name macros
 
-// TODO:
-// Some of the assertions in this test are failing on Apple platforms.
-// Until we figure out the problem and fix it, disable these tests on
-// Apple platforms. Note that we're not using XFAIL or UNSUPPORTED markup
-// here, because this test would otherwise be disabled on all platforms
-// we test. To avoid this test becoming entirely stale, we just disable
-// the parts that fail.
-//
-// See https://llvm.org/PR45739 for the bug tracking this.
-#if defined(__APPLE__)
-#   define APPLE_FIXME
-#endif
-
 typedef std::money_get<char, cpp17_input_iterator<const char*> > Fn;
 
 class my_facet
@@ -69,7 +50,11 @@ public:
     explicit my_facetw(std::size_t refs = 0)
         : Fw(refs) {}
 };
-#endif
+
+static std::wstring convert_thousands_sep(std::wstring const& in) {
+  return LocaleHelpers::convert_thousands_sep_ru_RU(in);
+}
+#endif // TEST_HAS_NO_WIDE_CHARACTERS
 
 int main(int, char**)
 {
@@ -85,6 +70,7 @@ int main(int, char**)
     ios.imbue(std::locale(ios.getloc(),
                           new std::moneypunct_byname<wchar_t, true>(loc_name)));
 #endif
+    std::string symbol(LocaleHelpers::currency_symbol_ru_RU());
     {
         const my_facet f(1);
         // char, national
@@ -95,7 +81,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 0);
         }
@@ -106,7 +92,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -1);
         }
@@ -117,7 +103,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 123456789);
         }
@@ -128,7 +114,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
         }
@@ -139,115 +125,115 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
         }
         {   // zero, showbase
-            std::string v = "0,00 \xD1\x80\xD1\x83\xD0\xB1"".";
+            std::string v = "0,00 " + symbol;
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 5);
+            assert(base(iter) == v.data() + 5);
             assert(err == std::ios_base::goodbit);
             assert(ex == 0);
         }
         {   // zero, showbase
-            std::string v = "0,00 \xD1\x80\xD1\x83\xD0\xB1"".";
-            showbase(ios);
+            std::string v = "0,00 " + symbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 0);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative one, showbase
-            std::string v = "-0,01 \xD1\x80\xD1\x83\xD0\xB1"".";
+            std::string v = "-0,01 " + symbol;
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 6);
+            assert(base(iter) == v.data() + 6);
             assert(err == std::ios_base::goodbit);
             assert(ex == -1);
         }
         {   // negative one, showbase
-            std::string v = "-0,01 \xD1\x80\xD1\x83\xD0\xB1"".";
-            showbase(ios);
+            std::string v = "-0,01 " + symbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -1);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // positive, showbase
-            std::string v = "1 234 567,89 \xD1\x80\xD1\x83\xD0\xB1"".";
+            std::string v = "1 234 567,89 " + symbol;
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 13);
+            assert(base(iter) == v.data() + 13);
             assert(err == std::ios_base::goodbit);
             assert(ex == 123456789);
         }
         {   // positive, showbase
-            std::string v = "1 234 567,89 \xD1\x80\xD1\x83\xD0\xB1"".";
-            showbase(ios);
+            std::string v = "1 234 567,89 " + symbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 123456789);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative, showbase
-            std::string v = "-1 234 567,89 \xD1\x80\xD1\x83\xD0\xB1"".";
-            showbase(ios);
+            std::string v = "-1 234 567,89 " + symbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative, showbase
-            std::string v = "-1 234 567,89 RUB ";
-            showbase(ios);
+            std::string v = "-1 234 567,89 RUB";
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 14);
+            assert(base(iter) == v.data() + 14);
             assert(err == std::ios_base::failbit);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative, showbase
-            std::string v = "-1 234 567,89 RUB ";
+            std::string v = "-1 234 567,89 RUB";
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 14);
+            assert(base(iter) == v.data() + 14);
             assert(err == std::ios_base::goodbit);
             assert(ex == -123456789);
         }
@@ -262,7 +248,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 0);
         }
@@ -273,7 +259,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -1);
         }
@@ -284,7 +270,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 123456789);
         }
@@ -295,7 +281,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
         }
@@ -306,128 +292,121 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
         }
         {   // zero, showbase
-            std::string v = "0,00 RUB ";
+            std::string v = "0,00 RUB";
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 5);
+            assert(base(iter) == v.data() + 5);
             assert(err == std::ios_base::goodbit);
             assert(ex == 0);
         }
-#if !defined(APPLE_FIXME)
         {   // zero, showbase
-            std::string v = "0,00 RUB ";
-            showbase(ios);
+            std::string v = "0,00 RUB";
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 0);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
-#endif
         {   // negative one, showbase
-            std::string v = "-0,01 RUB ";
+            std::string v = "-0,01 RUB";
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 6);
+            assert(base(iter) == v.data() + 6);
             assert(err == std::ios_base::goodbit);
             assert(ex == -1);
         }
-#if !defined(APPLE_FIXME)
         {   // negative one, showbase
-            std::string v = "-0,01 RUB ";
-            showbase(ios);
+            std::string v = "-0,01 RUB";
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -1);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
-#endif
         {   // positive, showbase
-            std::string v = "1 234 567,89 RUB ";
+            std::string v = "1 234 567,89 RUB";
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 13);
+            assert(base(iter) == v.data() + 13);
             assert(err == std::ios_base::goodbit);
             assert(ex == 123456789);
         }
-#if !defined(APPLE_FIXME)
         {   // positive, showbase
-            std::string v = "1 234 567,89 RUB ";
-            showbase(ios);
+            std::string v = "1 234 567,89 RUB";
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 123456789);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
-#endif
-#if !defined(APPLE_FIXME)
         {   // negative, showbase
-            std::string v = "-1 234 567,89 RUB ";
-            showbase(ios);
+            std::string v = "-1 234 567,89 RUB";
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
-#endif
         {   // negative, showbase
-            std::string v = "-1 234 567,89 \xD1\x80\xD1\x83\xD0\xB1"".";
-            showbase(ios);
+            std::string v = "-1 234 567,89 " + symbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 14);
+            assert(base(iter) == v.data() + 14);
             assert(err == std::ios_base::failbit);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative, showbase
-            std::string v = "-1 234 567,89 \xD1\x80\xD1\x83\xD0\xB1"".";
+            std::string v = "-1 234 567,89 " + symbol;
             typedef cpp17_input_iterator<const char*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 14);
+            assert(base(iter) == v.data() + 14);
             assert(err == std::ios_base::goodbit);
             assert(ex == -123456789);
         }
     }
 #ifndef TEST_HAS_NO_WIDE_CHARACTERS
+    std::wstring wsymbol(LocaleHelpers::currency_symbol_ru_RU());
     {
         const my_facetw f(1);
         // wchar_t, national
@@ -438,7 +417,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 0);
         }
@@ -449,29 +428,29 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -1);
         }
         {   // positive
-            std::wstring v = L"1 234 567,89 ";
+            std::wstring v = convert_thousands_sep(L"1 234 567,89 ");
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 123456789);
         }
         {   // negative
-            std::wstring v = L"-1 234 567,89 ";
+            std::wstring v = convert_thousands_sep(L"-1 234 567,89 ");
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
         }
@@ -482,115 +461,115 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
         }
         {   // zero, showbase
-            std::wstring v = L"0,00 \x440\x443\x431"".";
+            std::wstring v = L"0,00 " + wsymbol;
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 5);
+            assert(base(iter) == v.data() + 5);
             assert(err == std::ios_base::goodbit);
             assert(ex == 0);
         }
         {   // zero, showbase
-            std::wstring v = L"0,00 \x440\x443\x431"".";
-            showbase(ios);
+            std::wstring v = L"0,00 " + wsymbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 0);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative one, showbase
-            std::wstring v = L"-0,01 \x440\x443\x431"".";
+            std::wstring v = L"-0,01 " + wsymbol;
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 6);
+            assert(base(iter) == v.data() + 6);
             assert(err == std::ios_base::goodbit);
             assert(ex == -1);
         }
         {   // negative one, showbase
-            std::wstring v = L"-0,01 \x440\x443\x431"".";
-            showbase(ios);
+            std::wstring v = L"-0,01 " + wsymbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -1);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // positive, showbase
-            std::wstring v = L"1 234 567,89 \x440\x443\x431"".";
+            std::wstring v = convert_thousands_sep(L"1 234 567,89 ") + wsymbol;
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 13);
+            assert(base(iter) == v.data() + 13);
             assert(err == std::ios_base::goodbit);
             assert(ex == 123456789);
         }
         {   // positive, showbase
-            std::wstring v = L"1 234 567,89 \x440\x443\x431"".";
-            showbase(ios);
+            std::wstring v = convert_thousands_sep(L"1 234 567,89 ") + wsymbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 123456789);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative, showbase
-            std::wstring v = L"-1 234 567,89 \x440\x443\x431"".";
-            showbase(ios);
+            std::wstring v = convert_thousands_sep(L"-1 234 567,89 ") + wsymbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative, showbase
-            std::wstring v = L"-1 234 567,89 RUB ";
-            showbase(ios);
+            std::wstring v = convert_thousands_sep(L"-1 234 567,89 RUB");
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 14);
+            assert(base(iter) == v.data() + 14);
             assert(err == std::ios_base::failbit);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative, showbase
-            std::wstring v = L"-1 234 567,89 RUB ";
+            std::wstring v = convert_thousands_sep(L"-1 234 567,89 RUB");
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 false, ios, err, ex);
-            assert(iter.base() == v.data() + 14);
+            assert(base(iter) == v.data() + 14);
             assert(err == std::ios_base::goodbit);
             assert(ex == -123456789);
         }
@@ -605,7 +584,7 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 0);
         }
@@ -616,29 +595,29 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -1);
         }
         {   // positive
-            std::wstring v = L"1 234 567,89 ";
+            std::wstring v = convert_thousands_sep(L"1 234 567,89 ");
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 123456789);
         }
         {   // negative
-            std::wstring v = L"-1 234 567,89 ";
+            std::wstring v = convert_thousands_sep(L"-1 234 567,89 ");
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
         }
@@ -649,123 +628,115 @@ int main(int, char**)
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
         }
         {   // zero, showbase
-            std::wstring v = L"0,00 RUB ";
+            std::wstring v = L"0,00 RUB";
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 5);
+            assert(base(iter) == v.data() + 5);
             assert(err == std::ios_base::goodbit);
             assert(ex == 0);
         }
-#if !defined(APPLE_FIXME)
         {   // zero, showbase
-            std::wstring v = L"0,00 RUB ";
-            showbase(ios);
+            std::wstring v = L"0,00 RUB";
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 0);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
-#endif
         {   // negative one, showbase
-            std::wstring v = L"-0,01 RUB ";
+            std::wstring v = L"-0,01 RUB";
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 6);
+            assert(base(iter) == v.data() + 6);
             assert(err == std::ios_base::goodbit);
             assert(ex == -1);
         }
-#if !defined(APPLE_FIXME)
         {   // negative one, showbase
-            std::wstring v = L"-0,01 RUB ";
-            showbase(ios);
+            std::wstring v = L"-0,01 RUB";
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -1);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
-#endif
         {   // positive, showbase
-            std::wstring v = L"1 234 567,89 RUB ";
+            std::wstring v = convert_thousands_sep(L"1 234 567,89 RUB");
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 13);
+            assert(base(iter) == v.data() + 13);
             assert(err == std::ios_base::goodbit);
             assert(ex == 123456789);
         }
-#if !defined(APPLE_FIXME)
         {   // positive, showbase
-            std::wstring v = L"1 234 567,89 RUB ";
-            showbase(ios);
+            std::wstring v = convert_thousands_sep(L"1 234 567,89 RUB");
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == 123456789);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
-#endif
-#if !defined(APPLE_FIXME)
         {   // negative, showbase
-            std::wstring v = L"-1 234 567,89 RUB ";
-            showbase(ios);
+            std::wstring v = convert_thousands_sep(L"-1 234 567,89 RUB");
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + v.size());
+            assert(base(iter) == v.data() + v.size());
             assert(err == std::ios_base::eofbit);
             assert(ex == -123456789);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
-#endif
         {   // negative, showbase
-            std::wstring v = L"-1 234 567,89 \x440\x443\x431"".";
-            showbase(ios);
+            std::wstring v = convert_thousands_sep(L"-1 234 567,89 ") + wsymbol;
+            std::showbase(ios);
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 14);
+            assert(base(iter) == v.data() + 14);
             assert(err == std::ios_base::failbit);
-            noshowbase(ios);
+            std::noshowbase(ios);
         }
         {   // negative, showbase
-            std::wstring v = L"-1 234 567,89 \x440\x443\x431"".";
+            std::wstring v = convert_thousands_sep(L"-1 234 567,89 ") + wsymbol;
             typedef cpp17_input_iterator<const wchar_t*> I;
             long double ex;
             std::ios_base::iostate err = std::ios_base::goodbit;
             I iter = f.get(I(v.data()), I(v.data() + v.size()),
                                                 true, ios, err, ex);
-            assert(iter.base() == v.data() + 14);
+            assert(base(iter) == v.data() + 14);
             assert(err == std::ios_base::goodbit);
             assert(ex == -123456789);
         }
