@@ -8,6 +8,4 @@
 
 #include <string_view>
 
-int main(int, char**) {
-  return 0;
-}
+int main(int, char**) { return 0; }
