@@ -30,9 +30,7 @@ void test0(S s, U val, S expected, std::size_t expected_erased_count) {
 }
 
 template <class S>
-void test()
-{
-
+void test() {
   test0(S(""), 'a', S(""), 0);
 
   test0(S("a"), 'a', S(""), 1);
@@ -66,11 +64,10 @@ void test()
   test0(S("aba"), opt('c'), S("aba"), 0);
 }
 
-int main(int, char**)
-{
-    test<std::string>();
-    test<std::basic_string<char, std::char_traits<char>, min_allocator<char>>> ();
-    test<std::basic_string<char, std::char_traits<char>, test_allocator<char>>> ();
+int main(int, char**) {
+  test<std::string>();
+  test<std::basic_string<char, std::char_traits<char>, min_allocator<char>>>();
+  test<std::basic_string<char, std::char_traits<char>, test_allocator<char>>>();
 
   return 0;
 }
