@@ -7,9 +7,6 @@ Provides the configuration class, which holds all information related to
 how this invocation of the test suite should be run.
 """
 
-from __future__ import absolute_import
-from __future__ import print_function
-
 # System modules
 import os
 
@@ -62,7 +59,7 @@ yaml2obj = None
 # The arch might dictate some specific CFLAGS to be passed to the toolchain to build
 # the inferior programs.  The global variable cflags_extras provides a hook to do
 # just that.
-cflags_extras = ''
+cflags_extras = ""
 
 # The filters (testclass.testmethod) used to admit tests into our test suite.
 filters = []
@@ -78,7 +75,7 @@ xfail_tests = None
 # Set this flag if there is any session info dumped during the test run.
 sdir_has_content = False
 # svn_info stores the output from 'svn info lldb.base.dir'.
-svn_info = ''
+svn_info = ""
 
 # Default verbosity is 0.
 verbose = 0
@@ -93,7 +90,7 @@ testdirs = [lldbsuite.lldb_test_root]
 test_src_root = lldbsuite.lldb_test_root
 
 # Separator string.
-separator = '-' * 70
+separator = "-" * 70
 
 failed = False
 
@@ -133,8 +130,10 @@ enabled_plugins = []
 
 def shouldSkipBecauseOfCategories(test_categories):
     if use_categories:
-        if len(test_categories) == 0 or len(
-                categories_list & set(test_categories)) == 0:
+        if (
+            len(test_categories) == 0
+            or len(categories_list & set(test_categories)) == 0
+        ):
             return True
 
     for category in skip_categories:
@@ -151,6 +150,7 @@ def get_filecheck_path():
     if filecheck and os.path.lexists(filecheck):
         return filecheck
 
+
 def get_yaml2obj_path():
     """
     Get the path to the yaml2obj tool.
